//********************************************************************************
//file:		safe_queue.c
//author:	Xu Jianping
//description: a queue for UART
//Create Time:2021-10-15
//********************************************************************************

#include "sys.h"
#include "safe_queue.h"

void QInit(SafeQueue* pQueue, u8_t iPort, u16_t nQSize, u8_t* pDataBuffer, u8_t bRTSCTS)
{
	pQueue->iPort	= iPort; 	//0xFF = not UART 

	pQueue->bFlow	= bRTSCTS;	//0 = no modem 
	pQueue->nSize	= nQSize;
	pQueue->nHead	= 0;
	pQueue->nTail	= 0;

	pQueue->bDirectSend 	= 1;
	pQueue->bLastCTS		= 0;

	pQueue->cbDirectSend	= 0;
	pQueue->cbIsTHRE		= 0;
	pQueue->cbIsCTS			= 0;
	pQueue->cbSetRTS		= 0;
	pQueue->cbClrRTS		= 0;

	pQueue->cbSetRxSem		= 0;
	pQueue->cbSetTxSem		= 0;
	
	pQueue->pQueue = pDataBuffer;
}

void QReset(SafeQueue* pQueue, u8_t bInISR)
{
	LOCK_DEF
	if (!bInISR) LOCK_SET
		pQueue->nHead	= 0;
		pQueue->nTail	= 0;
	if (!bInISR) LOCK_CLR
}

u8_t QIsEmpty(SafeQueue* pQueue, u8_t bInISR)
{
	u8_t bEmpty;
	
	LOCK_DEF
	if (!bInISR) LOCK_SET
		bEmpty = (pQueue->nHead == pQueue->nTail);
	if (!bInISR) LOCK_CLR
	
	return bEmpty;
}

u8_t QIsHigh(SafeQueue* pQueue, u8_t bInISR)
{
	u8_t bFull;
	u16_t nLength;
	
	LOCK_DEF
	if (!bInISR) LOCK_SET
		nLength =   (pQueue->nHead >= pQueue->nTail) ? \
								(pQueue->nHead - pQueue->nTail)  : \
								(pQueue->nSize - pQueue->nTail + pQueue->nHead);
		bFull = (nLength > (3*pQueue->nSize/4));
	if (!bInISR) LOCK_CLR
	
	return bFull;
}

u8_t QIsFull(SafeQueue* pQueue, u8_t bInISR)
{
	u8_t bFull;
	
	LOCK_DEF
	if (!bInISR) LOCK_SET
		bFull = (((pQueue->nHead+1) % pQueue->nSize) == pQueue->nTail);
	if (!bInISR) LOCK_CLR
	
	return bFull;
}

u16_t QLength(SafeQueue* pQueue, u8_t bInISR)
{
	u32_t nLength;

	LOCK_DEF
	if (!bInISR) LOCK_SET
		nLength =   (pQueue->nHead >= pQueue->nTail) ? \
								(pQueue->nHead - pQueue->nTail)  : \
								(pQueue->nSize - pQueue->nTail + pQueue->nHead);
	if (!bInISR) LOCK_CLR

	return nLength;
}

u16_t QFreeLen(SafeQueue* pQueue, u8_t bInISR)
{
	u16_t nFreeLen;

	LOCK_DEF
	if (!bInISR) LOCK_SET
		nFreeLen =  (pQueue->nHead >= pQueue->nTail) ? \
								(pQueue->nSize - (pQueue->nHead - pQueue->nTail) - 1) : \
								(pQueue->nTail - pQueue->nHead - 1);
	if (!bInISR) LOCK_CLR

	return nFreeLen;
}

u16_t QInBlock(SafeQueue* pQueue, u8_t* pData, u16_t nLen, u8_t bInISR, u8_t EnableSplit)
{
	u16_t i;
	u16_t nBytes = 0;
	u16_t nQFree, nQLen;
	
	LOCK_DEF
	if (!bInISR) LOCK_SET
	
		nQLen = (pQueue->nHead >= pQueue->nTail) ? 
						(pQueue->nHead - pQueue->nTail) : 
						(pQueue->nSize - pQueue->nTail + pQueue->nHead);
		nQFree = pQueue->nSize - nQLen - 1;
		
//		if ((EnableSplit == 0) && (nQFree < nLen))
//		{
//			if (!bInISR) LOCK_CLR
//			return 0;
//		}
		
		nBytes = min(nQFree, nLen);
		for(i = 0; i < nBytes; i ++)
		{
			pQueue->pQueue[pQueue->nHead] = pData[i];
			pQueue->nHead ++;
			pQueue->nHead %= pQueue->nSize;
		}
		nQLen += nBytes;
		nQFree -= nBytes;
	
		if (pQueue->iPort != 0xFF)
		{
			if (bInISR == 0)
			{ //send by app. task
				if ((nQLen != 0) && 
						((pQueue->bFlow == 0) || ((pQueue->cbIsCTS != NULL)?pQueue->cbIsCTS(pQueue->iPort):1)))
				{
					if (pQueue->bDirectSend != 0)  	//direct send
					{ //send only, direct send to trigger the xmit interrupt
						pQueue->bDirectSend = 0;
						
						if (pQueue->cbDirectSend != NULL)
						{
							pQueue->cbDirectSend(pQueue->iPort, &pQueue->pQueue[pQueue->nTail], 1, bInISR!=0);
						
							pQueue->nTail ++;
							pQueue->nTail %= pQueue->nSize;
							
							nQLen --;
						}
					}
				}
			}
			else if ((bInISR == 1) && (pQueue->bFlow == 1) && (nQLen > (3*pQueue->nSize/4)))
			{ //receive in ISR only, 
				if (pQueue->cbClrRTS != NULL)
					pQueue->cbClrRTS(pQueue->iPort);	//let the peer stop, if Rx buffer is high level
			}
		}
		
	if (!bInISR) LOCK_CLR
		
	if ((nBytes > 0) && (pQueue->cbSetRxSem != NULL))
	{
		pQueue->cbSetRxSem(pQueue->iPort);
	}
	
	return nBytes;
}

u16_t QOutBlock(SafeQueue* pQueue, u8_t* pData, u16_t nLen, u8_t bInISR)
{
	u16_t i;
	u16_t nBytes = 0;
	u16_t nQLen;
	
	LOCK_DEF
	if (!bInISR) LOCK_SET

		nQLen = (pQueue->nHead >= pQueue->nTail) ? 
						(pQueue->nHead - pQueue->nTail) : 
						(pQueue->nSize - pQueue->nTail + pQueue->nHead);
		nBytes = min(nQLen, nLen);
		
		if (pQueue->iPort != 0xFF)
		{
			if (bInISR == 1)
			{ //send in ISR
				if ((pQueue->bFlow == 1) && !((pQueue->cbIsCTS != NULL)?pQueue->cbIsCTS(pQueue->iPort):1))
				{
					nBytes = 0; 				//can't send if CTS is not set
				}
				if (nBytes == 0)
					pQueue->bDirectSend = 1;	//nothing to send, and next byte should be sent to port
			}
		}

		for(i = 0; i < nBytes; i ++)
		{
			pData[i] = pQueue->pQueue[pQueue->nTail];
			pQueue->nTail ++;
			pQueue->nTail %= pQueue->nSize;
		}

		if (pQueue->iPort != 0xFF)
		{
			if ((bInISR == 0) && (pQueue->bFlow == 1))
			{ //read by app. thread 
				nQLen -= nBytes;
				if (nQLen < (pQueue->nSize/4))
				{ //Rx buffer is low level, the peer may stop sending, Need a RTS request
					if (pQueue->cbSetRTS != NULL)
						pQueue->cbSetRTS(pQueue->iPort);
				}
			}
		}
		
	if (!bInISR) LOCK_CLR

	if ((nBytes > 0) && (pQueue->cbSetTxSem != NULL))
	{
		pQueue->cbSetTxSem(pQueue->iPort);
	}

	return nBytes;
}
